/*
	Created by Jesse R. Warden a.k.a. "JesterXL"
	jesterxl@jessewarden.com
	http://www.jessewarden.com
	http://www.universalmind.com
	
	This is release under a Creative Commons license. 
    More information can be found here:
    
    http://creativecommons.org/licenses/by/2.5/
*/
package com.jxl.downloads.views
{
	import mx.core.UIComponent;
	import mx.states.State;
	
	import com.jxl.downloads.views.BackgroundPanelSymbol;
	import mx.events.FlexEvent;
	import flash.events.Event;
	
	[Event(name="introStateDone", type="mx.events.FlexEvent")]
	[Event(name="spinStateDone", type="mx.events.FlexEvent")]
	public class BackgroundPanel extends UIComponent
	{
		
		public static const STATE_HIDDEN:String 		= "hiddenState";
		public static const STATE_INTRO:String 			= "introState";
		public static const STATE_MAIN:String 			= "mainState";
		public static const STATE_SPIN:String 			= "spinState";
		
		public static const INTRO_STATE_DONE:String 	= "introStateDone";
		public static const SPIN_STATE_DONE:String		= "spinStateDone";
		
		protected var __hidden_state:State;
		protected var __intro_state:State;
		protected var __main_state:State;
		protected var __spin_state:State;
		
		protected var __bgSymbol:BackgroundPanelSymbol;
		
		protected var __isFirstTimeShownIntro:Boolean 	= true;
		protected var __isFirstTimeShownSpin:Boolean	= true;
		
		public function BackgroundPanel()
		{
			super();
			
			__hidden_state = new State();
			__hidden_state.name = STATE_HIDDEN;
			__hidden_state.addEventListener(FlexEvent.ENTER_STATE, onHiddenStateEnter);
			states.push(__hidden_state);
			
			__intro_state = new State();
			__intro_state.name = STATE_INTRO;
			__intro_state.addEventListener(FlexEvent.ENTER_STATE, onIntroStateEnter);
			states.push(__intro_state);
			
			__main_state = new State();
			__main_state.name = STATE_MAIN;
			__main_state.addEventListener(FlexEvent.ENTER_STATE, onMainStateEnter);
			states.push(__main_state);
			
			__spin_state = new State();
			__spin_state.name = STATE_SPIN;
			__spin_state.addEventListener(FlexEvent.ENTER_STATE, onSpinStateEnter);
			states.push(__spin_state);
			
			currentState = STATE_HIDDEN;
			
		}
		
		protected override function createChildren():void
		{
			super.createChildren();
			
			if(__bgSymbol == null)
			{
				__bgSymbol = new BackgroundPanelSymbol();
				addChild(__bgSymbol);
				
				__bgSymbol.addEventListener(BackgroundPanelSymbol.DONE_INTRO, onDoneTransitioning);
				__bgSymbol.addEventListener(BackgroundPanelSymbol.DONE_SPIN, onDoneTransitioning);
				__bgSymbol.addEventListener(BackgroundPanelSymbol.DONE_INTRO_PRELOADING, onDoneTransitioning);
				__bgSymbol.addEventListener(BackgroundPanelSymbol.DONE_SPIN_PRELOADING, onDoneTransitioning);
			}
		}
		
		protected function onHiddenStateEnter(p_event:FlexEvent):void
		{
			__bgSymbol.hidden();
		}
		
		protected function onIntroStateEnter(p_event:FlexEvent):void
		{
			if(__isFirstTimeShownIntro == false)
			{
				__bgSymbol.showIntro();
			}
			else
			{
				__bgSymbol.preloadIntro();
			}
		}
		
		protected function onMainStateEnter(p_event:FlexEvent):void
		{
			__bgSymbol.showMain();
		}
		
		protected function onSpinStateEnter(p_event:FlexEvent):void
		{
			if(__isFirstTimeShownSpin == false)
			{
				__bgSymbol.showSpin();
			}
			else
			{
				__bgSymbol.preloadSpin();
			}
		}
		
		protected function onDoneTransitioning(p_event:Event):void
		{
			DebugWindow.debugHeader();
			DebugWindow.debug("BackgroundPanel::onDoneTransitioning");
			DebugWindow.debug("p_event.type: " + p_event.type);
			switch(p_event.type)
			{
				case BackgroundPanelSymbol.DONE_INTRO:
					dispatchEvent(new FlexEvent(INTRO_STATE_DONE));
					currentState = STATE_MAIN;
					break;
				
				case BackgroundPanelSymbol.DONE_SPIN:
					dispatchEvent(new FlexEvent(SPIN_STATE_DONE));
					currentState = STATE_MAIN;
					break;
					
				case BackgroundPanelSymbol.DONE_INTRO_PRELOADING:
					__isFirstTimeShownIntro = false;
					__bgSymbol.showIntro();
					break;
					
				case BackgroundPanelSymbol.DONE_SPIN_PRELOADING:
					__isFirstTimeShownSpin = false;
					__bgSymbol.showSpin();
					break;
			}
		}
	}
}