/*
	Created by Jesse R. Warden a.k.a. "JesterXL"
	jesterxl@jessewarden.com
	http://www.jessewarden.com
	http://www.universalmind.com
	
	This is release under a Creative Commons license. 
    More information can be found here:
    
    http://creativecommons.org/licenses/by/2.5/
*/
package com.jxl.downloads.views
{
	import mx.containers.Canvas;
	import mx.controls.Image;
	import com.jxl.downloads.views.BackgroundPanel;
	import com.jxl.downloads.views.PanelButtons;
	import com.jxl.downloads.views.Skies;
	import mx.events.FlexEvent;
	import flash.events.MouseEvent;
	
	// all yer base are belong to Canvas
	[Event(name="spinStateDone", type="mx.events.FlexEvent")]
	[Event(name="introStateDone", type="mx.events.FlexEvent")]
	public class BaseDownloadsPanel extends Canvas
	{	
		public static const SPIN_STATE_DONE:String 				= "spinStateDone";
		public static const INTRO_STATE_DONE:String 			= "introStateDone";
		
		public static const BUTTON_CLICK_ABOUT_US:String 		= "aboutUsButtonClicked";
		public static const BUTTON_CLICK_SERVICES:String 		= "servicesButtonClicked";
		public static const BUTTON_CLICK_PORTFOLIO:String 		= "portfoliButtonClicked";
		public static const BUTTON_CLICK_PRICING:String 		= "pricingButtonClicked";
		public static const BUTTON_CLICK_CONTACTS:String 		= "contactsButtonClicked";
		
		[Embed(source="../../../../assets/images/background.png")]
		public static const BACKGROUND_IMAGE:Class;
		
		protected static const BASE_STATE_HIDDEN:String 	= "com.jxl.downloads.views.BaseDownloadsPanel.hiddenState";
		protected static const BASE_STATE_INTRO:String 		= "com.jxl.downloads.views.BaseDownloadsPanel.introState";
		protected static const BASE_STATE_MAIN:String 		= "com.jxl.downloads.views.BaseDownloadsPanel.mainState";
		protected static const BASE_STATE_SPIN:String 		= "com.jxl.downloads.views.BaseDownloadsPanel.spinState";
		
		protected var __bgImage:Image;
		protected var __skies:Skies;
		protected var __bgPanel:BackgroundPanel;
		protected var __panelButtons:PanelButtons;
		protected var __theCurrenState:String = BASE_STATE_HIDDEN;
		
		public function BaseDownloadsPanel()
		{
			super();
		}
		
		public function hidden():void
		{
			__theCurrenState = BASE_STATE_HIDDEN;
			
			__skies.currentState = Skies.STATE_HIDDEN;
			__bgPanel.currentState = BackgroundPanel.STATE_HIDDEN;
			
			hideChildren();
		}
		
		public function showIntro():void
		{
			__theCurrenState = BASE_STATE_INTRO;
			
			showChildren();
			
			__skies.currentState = Skies.STATE_HIDDEN;
			__panelButtons.currentState = PanelButtons.STATE_HIDDEN;
			__bgPanel.currentState = BackgroundPanel.STATE_INTRO;
		}
		
		public function showMain():void
		{
			__theCurrenState = BASE_STATE_MAIN;
			
			showChildren();
			
			__skies.currentState = Skies.STATE_MAIN;
			__bgPanel.currentState = BackgroundPanel.STATE_MAIN;
			__panelButtons.currentState = PanelButtons.STATE_MAIN;
		}
		
		public function spinSpinSugar():void
		{
			__theCurrenState = BASE_STATE_SPIN;
			
			showChildren();
			
			__skies.currentState = Skies.STATE_FADE_OUT;
			__panelButtons.currentState = PanelButtons.STATE_DISAPPEAR;
		}
		
		protected override function createChildren():void
		{
			super.createChildren();
			
			if(__bgImage == null)
			{
				__bgImage = new Image();
				__bgImage.source = BACKGROUND_IMAGE;
			}
			
			if(__skies == null)
			{
				__skies = new Skies();
			}
			
			if(__bgPanel == null)
			{
				__bgPanel = new BackgroundPanel();
				__bgPanel.addEventListener(BackgroundPanel.INTRO_STATE_DONE, 
											onBackgroundPanelIntroStateDone);
				__bgPanel.addEventListener(BackgroundPanel.SPIN_STATE_DONE, 
											onBackgroundPanelSpinStateDone);
			}
			
			if(__panelButtons == null)
			{
				__panelButtons = new PanelButtons();
				__panelButtons.addEventListener(PanelButtons.APPEAR_STATE_DONE,
												onPanelButtonsAppearStateDone);
				__panelButtons.addEventListener(PanelButtons.DISAPPEAR_STATE_DONE,
												onPanelButtonsDisappearStateDone);
				__panelButtons.addEventListener(PanelButtons.BUTTON_CLICK_ABOUT_US,
												onPanelButtonClicked);
				__panelButtons.addEventListener(PanelButtons.BUTTON_CLICK_SERVICES,
												onPanelButtonClicked);
				__panelButtons.addEventListener(PanelButtons.BUTTON_CLICK_PORTFOLIO,
												onPanelButtonClicked);
				__panelButtons.addEventListener(PanelButtons.BUTTON_CLICK_PRICING,
												onPanelButtonClicked);
				__panelButtons.addEventListener(PanelButtons.BUTTON_CLICK_CONTACTS,
												onPanelButtonClicked);			
			}
		}
		
		protected override function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			super.updateDisplayList(unscaledWidth, unscaledHeight);
			
			__panelButtons.move(50, 90);
			
		}
		
		protected function onBackgroundPanelIntroStateDone(p_event:FlexEvent):void
		{
			__skies.currentState = Skies.STATE_FADE_IN;
			__panelButtons.currentState = PanelButtons.STATE_APPEAR;
		}
		
		protected function onBackgroundPanelSpinStateDone(p_event:FlexEvent):void
		{
			__skies.currentState = Skies.STATE_FADE_IN;
			__panelButtons.currentState = PanelButtons.STATE_APPEAR;
		}
		
		protected function onPanelButtonsAppearStateDone(p_event:FlexEvent):void
		{
			if(__theCurrenState == BASE_STATE_SPIN)
			{
				onSpinStateDone();
				dispatchEvent(new FlexEvent(SPIN_STATE_DONE));
			}
			else if(__theCurrenState == BASE_STATE_INTRO)
			{
				onIntroStateDone();
				dispatchEvent(new FlexEvent(INTRO_STATE_DONE));
			}
		}
		
		protected function onPanelButtonsDisappearStateDone(p_event:FlexEvent):void
		{
			if(__theCurrenState == BASE_STATE_SPIN)
				__bgPanel.currentState = BackgroundPanel.STATE_SPIN;
		}
		
		protected function onPanelButtonClicked(p_event:FlexEvent):void
		{
			var event:FlexEvent = p_event.clone() as FlexEvent;
			dispatchEvent(event);
		}
		
		protected function hideChildren():void
		{
			if(contains(__bgImage)) 				removeChild(__bgImage);
			if(contains(__skies)) 					removeChild(__skies);
			if(contains(__bgPanel)) 				removeChild(__bgPanel);
			if(contains(__panelButtons)) 			removeChild(__panelButtons);
		}
		
		protected function showChildren():void
		{
			if(contains(__bgImage) == false) 		addChild(__bgImage);
			if(contains(__skies) == false) 			addChild(__skies);
			if(contains(__bgPanel) == false) 		addChild(__bgPanel);
			if(contains(__panelButtons) == false) 	addChild(__panelButtons);
			
			setChildIndex(__bgImage, 0);
			setChildIndex(__skies, 1);
			setChildIndex(__bgPanel, 2);
			setChildIndex(__panelButtons, 3);
		}
		
		protected function onIntroStateDone():void
		{
		}
		
		protected function onSpinStateDone():void
		{
		}
		
		
	}
}