/*
	Created by Jesse R. Warden a.k.a. "JesterXL"
	jesterxl@jessewarden.com
	http://www.jessewarden.com
	http://www.universalmind.com
	
	This is release under a Creative Commons license. 
    More information can be found here:
    
    http://creativecommons.org/licenses/by/2.5/
*/
package com.jxl.downloads.views
{
	import mx.core.UIComponent;
	import mx.states.State;
	
	import com.jxl.downloads.views.PanelButtonSymbol;
	import mx.events.FlexEvent;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	[Event(name="appearStateDone", type="mx.events.FlexEvent")]
	[Event(name="disappearStateDone", type="mx.events.FlexEvent")]
	public class PanelButton extends UIComponent
	{
		
		public static const STATE_HIDDEN:String 			= "hiddenState";
		public static const STATE_MAIN:String				= "mainState";
		public static const STATE_APPEAR:String 			= "appearState";
		public static const STATE_DISAPPEAR:String 			= "disappearState";
		
		public static const APPEAR_STATE_DONE:String		= "appearStateDone";
		public static const DISAPPEAR_STATE_DONE:String		= "disappearStateDone";
		
		
		protected var __hidden_state:State;
		protected var __main_state:State;
		protected var __appear_state:State;
		protected var __disappear_state:State;
		
		protected var __panelButton:PanelButtonSymbol;
		
		protected var __text:String;
		protected var __textDirty:Boolean = false;
		
		public function get text():String { return __text; }
		
		public function set text(p_val:String):void
		{
			__text = p_val;
			__textDirty = true;
			invalidateProperties();
		}
		
		public function PanelButton()
		{
			super();
			
			__hidden_state = new State();
			__hidden_state.name = STATE_HIDDEN;
			__hidden_state.addEventListener(FlexEvent.ENTER_STATE, onHiddenStateEnter);
			states.push(__hidden_state);
			
			__main_state = new State();
			__main_state.name = STATE_MAIN;
			__main_state.addEventListener(FlexEvent.ENTER_STATE, onMainStateEnter);
			states.push(__main_state);
			
			__appear_state = new State();
			__appear_state.name = STATE_APPEAR;
			__appear_state.addEventListener(FlexEvent.ENTER_STATE, onAppearStateEnter);
			states.push(__appear_state);
			
			__disappear_state = new State();
			__disappear_state.name = STATE_DISAPPEAR;
			__disappear_state.addEventListener(FlexEvent.ENTER_STATE, onDisappearStateEnter);
			states.push(__disappear_state);
			
			currentState = STATE_HIDDEN;
		}
		
		protected override function createChildren():void
		{
			super.createChildren();
			
			if(__panelButton == null)
			{
				__panelButton = new PanelButtonSymbol();
				addChild(__panelButton);
				
				__panelButton.addEventListener(PanelButtonSymbol.DONE_APPEARING, onDoneTransitioning);
				__panelButton.addEventListener(PanelButtonSymbol.DONE_DISAPPEARING, onDoneTransitioning);
				__panelButton.addEventListener(MouseEvent.CLICK, onPanelButtonClicked);
				__panelButton.setText(__text);
			}
		}
		
		protected override function commitProperties():void
		{
			super.commitProperties();
			
			if(__textDirty == true)
			{
				__textDirty = false;
				if(__panelButton != null)
				{
					__panelButton.setText(__text);
				}
			}
		}
		
		protected function onHiddenStateEnter(p_event:FlexEvent):void
		{
			__panelButton.hidden();
			if(contains(__panelButton)) removeChild(__panelButton);
		}
		
		protected function onMainStateEnter(p_event:FlexEvent):void
		{
			if(contains(__panelButton) == false) addChild(__panelButton);
			__panelButton.showMain();
		}
		
		protected function onAppearStateEnter(p_event:FlexEvent):void
		{
			if(contains(__panelButton) == false) addChild(__panelButton);
			__panelButton.appear();
		}
		
		protected function onDisappearStateEnter(p_event:FlexEvent):void
		{
			if(contains(__panelButton) == false) addChild(__panelButton);
			__panelButton.disappear();
		}
		
		protected function onDoneTransitioning(p_event:Event):void
		{
			switch(p_event.type)
			{
				case PanelButtonSymbol.DONE_APPEARING:
					dispatchEvent(new FlexEvent(APPEAR_STATE_DONE));
					currentState = STATE_MAIN;
					break;
				
				case PanelButtonSymbol.DONE_DISAPPEARING:
					dispatchEvent(new FlexEvent(DISAPPEAR_STATE_DONE));
					currentState = STATE_HIDDEN;
					break;
			}
		}
		
		protected function onPanelButtonClicked(p_event:MouseEvent):void
		{
			p_event.stopPropagation();
			var clickEvent:MouseEvent = new MouseEvent(MouseEvent.CLICK, false, true);
			dispatchEvent(clickEvent);
 		}
	}
}